package logs

import (
	"context"

	itemstransportgrpc "git.perx.ru/perxis/perxis-go/pkg/items/transport/grpc"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	pb "git.perx.ru/perxis/perxis-go/proto/logs"
)

const ServiceName = "logs"

type Service interface {

	// Log метод записи логов
	Log(ctx context.Context, entries []*Entry) error

	// Find метод для поиска логов по заданным параметрам
	Find(ctx context.Context, filter *Filter, options *options.FindOptions) (*FindResult, error)

	// Delete метод для удаления логов по заданным параметрам
	Delete(ctx context.Context, filter *Filter) error
}

type Filter struct {
	Q []string
}

type FindResult struct {
	Entries []*Entry
	Filter  *Filter
	Options *options.FindOptions
	Total   uint32
}

func FindResultToPB(result *FindResult) *pb.FindResult {
	findResult := &pb.FindResult{
		Total: result.Total,
	}

	entries := make([]*pb.LogEntry, 0, len(result.Entries))
	for _, e := range result.Entries {
		entries = append(entries, EntryToPB(e))
	}
	findResult.Entries = entries

	if result.Filter != nil {
		findResult.Filter.Q = result.Filter.Q
	}
	if result.Options != nil {
		findResult.Options, _ = itemstransportgrpc.PtrServicesFindOptionsToProto(result.Options)
	}

	return findResult
}

func FindResultFromPB(result *pb.FindResult) *FindResult {
	findResult := &FindResult{
		Total: result.Total,
	}

	entries := make([]*Entry, 0, len(result.Entries))
	for _, e := range result.Entries {
		entries = append(entries, EntryFromPB(e))
	}
	findResult.Entries = entries

	if result.Filter != nil {
		findResult.Filter.Q = result.Filter.Q
	}
	if result.Options != nil {
		findResult.Options, _ = itemstransportgrpc.ProtoToPtrServicesFindOptions(result.Options)
	}

	return findResult
}
