package items

import (
	"git.perx.ru/perxis/perxis-go/pkg/errors"
	pb "git.perx.ru/perxis/perxis-go/proto/items"
	"google.golang.org/protobuf/proto"
)

const (
	EventCreate    = "items.create"
	EventUpdate    = "items.update"
	EventPublish   = "items.publish"
	EventUnpublish = "items.unpublish"
	EventDelete    = "items.delete"
	EventUndelete  = "items.undelete"
	EventArchive   = "items.archive"
	EventUnarchive = "items.unarchive"

	DefaultEventSubject = "content.{{.EventType}}.{{.SpaceID}}.{{.EnvID}}.{{.CollectionID}}.{{.ItemID}}"
)

var (
	ErrInvalidEventType = func(expected string, got any) error {
		return errors.Errorf("invalid message type: expected '%s', got '%t'", expected, got)
	}
)

type EventCreateInfo struct {
	SpaceID      string
	EnvID        string
	CollectionID string
	ItemID       string
}

func (e EventCreateInfo) ToProto() (proto.Message, error) {
	return &pb.EventCreate{SpaceId: e.SpaceID, EnvId: e.EnvID, CollectionId: e.CollectionID, ItemId: e.ItemID}, nil
}

func (e *EventCreateInfo) FromProto(message proto.Message) error {
	p, ok := message.(*pb.EventCreate)
	if !ok {
		return ErrInvalidEventType("*pb.EventCreateInfo", message)
	}

	e.SpaceID = p.SpaceId
	e.EnvID = p.EnvId
	e.CollectionID = p.CollectionId
	e.ItemID = p.ItemId
	return nil
}

type EventUpdateInfo struct {
	SpaceID      string
	EnvID        string
	CollectionID string
	ItemID       string
}

func (e EventUpdateInfo) ToProto() (proto.Message, error) {
	return &pb.EventUpdate{SpaceId: e.SpaceID, EnvId: e.EnvID, CollectionId: e.CollectionID, ItemId: e.ItemID}, nil
}

func (e *EventUpdateInfo) FromProto(message proto.Message) error {
	p, ok := message.(*pb.EventUpdate)
	if !ok {
		return ErrInvalidEventType("*pb.EventUpdateInfo", message)
	}

	e.SpaceID = p.SpaceId
	e.EnvID = p.EnvId
	e.CollectionID = p.CollectionId
	e.ItemID = p.ItemId
	return nil
}

type EventPublishInfo struct {
	SpaceID      string
	EnvID        string
	CollectionID string
	ItemID       string
}

func (e EventPublishInfo) ToProto() (proto.Message, error) {
	return &pb.EventPublish{SpaceId: e.SpaceID, EnvId: e.EnvID, CollectionId: e.CollectionID, ItemId: e.ItemID}, nil
}

func (e *EventPublishInfo) FromProto(message proto.Message) error {
	p, ok := message.(*pb.EventPublish)
	if !ok {
		return ErrInvalidEventType("*pb.EventPublishInfo", message)
	}
	e.SpaceID = p.SpaceId
	e.EnvID = p.EnvId
	e.CollectionID = p.CollectionId
	e.ItemID = p.ItemId
	return nil
}

type EventUnpublishInfo struct {
	SpaceID      string
	EnvID        string
	CollectionID string
	ItemID       string
}

func (e EventUnpublishInfo) ToProto() (proto.Message, error) {
	return &pb.EventUnpublish{SpaceId: e.SpaceID, EnvId: e.EnvID, CollectionId: e.CollectionID, ItemId: e.ItemID}, nil
}

func (e *EventUnpublishInfo) FromProto(message proto.Message) error {
	p, ok := message.(*pb.EventUnpublish)
	if !ok {
		return ErrInvalidEventType("*pb.EventUnpublishInfo", message)
	}
	e.SpaceID = p.SpaceId
	e.EnvID = p.EnvId
	e.CollectionID = p.CollectionId
	e.ItemID = p.ItemId
	return nil
}

type EventDeleteInfo struct {
	SpaceID      string
	EnvID        string
	CollectionID string
	ItemID       string
}

func (e EventDeleteInfo) ToProto() (proto.Message, error) {
	return &pb.EventDelete{SpaceId: e.SpaceID, EnvId: e.EnvID, CollectionId: e.CollectionID, ItemId: e.ItemID}, nil
}

func (e *EventDeleteInfo) FromProto(message proto.Message) error {
	p, ok := message.(*pb.EventDelete)
	if !ok {
		return ErrInvalidEventType("*pb.EventDeleteInfo", message)
	}
	e.SpaceID = p.SpaceId
	e.EnvID = p.EnvId
	e.CollectionID = p.CollectionId
	e.ItemID = p.ItemId
	return nil
}
